#!/usr/bin/env python
# encoding: utf-8
"""
Mòdul que implementa la classe que representa una execució de l'algoritme ABC

Created by Joan T. Matamalas Llodrà on 2011-11-11.
Copyright (c) 2011 . All rights reserved.
"""

import sys
import getopt
import random
import time
from algorithms.ec import Ec

class Abc(Ec):
    """Mètode creador"""
    def __init__(self,function, seed, iterations, threshold=1e-10, nPop = 50):
        super(Abc, self).__init__(function, seed, iterations, threshold, nPop = 50)
        self.foodSrc = self.initPopulation() #Iniciam la població, Fonts d'aliment
        
    def fit(self, food):
        """Funció intermèdia que inverteix la tendència del fitness"""
        evaluation = self.function.evalf(food)
    
        if evaluation >= self.function.minimum:
            return 1/(1+evaluation)
        else:
            return 1 + abs(self.function.minimum)
    
        
    def onlookerFoodSelection(self):
        """Métode que determina les fonts d'aliment que s'assignaran a les abelles basant-nos amb el fitness"""
        #Probabilitat de ser seleccionat per una abella no ocupada
        totalFit = sum([self.fit(food) for food in self.foodSrc])
        p = [self.fit(food)/totalFit for food in self.foodSrc]
        
        #Omplint els resultats utilitzant el SUS
        onlookerFoodDestination = []
        r = random.uniform(0,(1/len(self.foodSrc)))
        sumProb = 0
        i = len(self.foodSrc)-1

        while len(onlookerFoodDestination) < len(self.foodSrc):
            sumProb += p[i]
            while r <= sumProb:
                onlookerFoodDestination.append(i)
                r += 1.0/len(self.foodSrc)
            i -= 1
  
        return onlookerFoodDestination
    
    def generateV(self, i):
        """Métode que calcula el desplaçament d'explotació d'una abella donada una font d'aliment"""
        j = random.choice(range(self.function.dimensions)) #Seleccionam la dimensió sobre la que desplaçarem l'abella
        k = i
        while k == i:
            k = random.choice(range(self.nPop)) #Seleccionam una font d'aliment diferent a l'actual la qual influira en el desplaçament
    
        #Factor d'escalat
        phi = random.uniform(-1,1)
        v = self.foodSrc[i][:]
            
        #Calcul del desplaçament
        newParameter = v[j] + phi*(v[j]-self.foodSrc[k][j])
            
        #Limitam els resultats dins els rangs desitjats
        if newParameter < self.function.rangeA:
            newParameter = self.function.rangeA
        elif newParameter > self.function.rangeB:
            newParameter = self.function.rangeB
            
        v[j] = newParameter
    
        return v

    def run(self):
        """Mètode que executa l'algoritme ABC"""
    
        self.startTime = time.time()
        
        #Inicialitzam el contador de nombre de explotacions sense millora de cada Font d'aliment
        foodTrialCount = [0]*self.nPop
        
        #Definim el límit de nombre de explotacions sense millora
        limit = self.nPop*self.function.dimensions
    
        #Guardam el millor individu
        self.setBestIndv(self.foodSrc)
            
        for iteration in range(self.iterations):
            #Employed bees phase
            for i in range(len(self.foodSrc)):
                #Calculam el desplaçament de la cerca.
                v = self.generateV(i)
            
                if self.fit(v) > self.fit(self.foodSrc[i]):
                    #Si la cerca ha trobat milloria actualitza les dades de la població i possa el contador a zero
                    self.foodSrc[i] = v[:]
                    foodTrialCount[i] = 0
                else:
                    #Si no n'ha trobada augmenta el contador d'aquesta font d'aliment en una unitat.
                    foodTrialCount[i] += 1
        
            #Onlooked bees phase
            onlookerFoodDestination = self.onlookerFoodSelection() #Destí de les abelles desocupades
        
            for i in onlookerFoodDestination:
                #Calculam el desplaçament de la cerca
                v = self.generateV(i) 
            
                if self.fit(v) > self.fit(self.foodSrc[i]):
                    #Si la cerca ha trobat milloria actualitza les dades de la població i possa el contador a zero
                    self.foodSrc[i] = v[:]
                    foodTrialCount[i] = 0
                else:
                    #Si no n'ha trobada augmenta el contador d'aquesta font d'aliment en una unitat.
                    foodTrialCount[i] += 1
                
            #Acutalizam els resultats
            self.updateResults(self.foodSrc)
            
            #Comprovam si s'ha arribat el mínim dessitjat
            if self.isGoalFounded: break
        
            #Scout phase
            #Si hi ha alguna font d'aliment que ha arribat al limit de cerca l'eliminam i en generam una de nova.
            for i in range(len(foodTrialCount)):
                if foodTrialCount[i] > limit:
                    self.foodSrc[i] = [random.uniform(self.function.rangeA, self.function.rangeB) for d in range(self.function.dimensions)]
                    foodTrialCount[i] = 0
    
        self.stopTime = time.time()