#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
"""
Mòdul que implementa la classe que representa una execució de l'algoritme DE

Created by Joan T. Matamalas Llodrà on 2011-11-11.
Copyright (c) 2011 . All rights reserved.
"""

import random
import math
import time
import os
import sys
from algorithms.ec import Ec

from functions import *

class De(Ec):
	"""Métode creador"""
	def __init__(self, function, seed, iterations, threshold=1e-10, nPop = 50, f = 0.4717, cr = 0.8803):
		super(De, self).__init__(function, seed, iterations, threshold, nPop = 50)
		self.f = f #Factor d'escalat del vector mutant
		self.cr = cr #Prioritat de l'individu original a l'hora de realitzar l'encreuament
		self.population = self.initPopulation() #Població de solucions candidates
	
	def run(self):
	
		#Inicialitzam el temps
		self.startTime = time.time()
	
		#Millor individu
		self.setBestIndv(self.population)
	
		#Iniciam les iteracions
		for iteration in range(self.iterations):
			nextPopulation = []
			
			for indv in self.population:
                #Es seleccionen 3 individus diferents.
				vectorsMut = random.sample(self.population,3)
				while indv in vectorsMut:
					vectorsMut = random.sample(self.population,3)
                    
                #Es genera el vector 'mutant' en base els 3 individus prèviament seleccionats.
				mutation = [(vectorsMut[0][i]+self.f*(vectorsMut[2][i]-vectorsMut[1][i])) for i in range(self.function.dimensions)]
				offspring = []
                
                #Crossover dimensió a dimensió del vector mutant amb l'individu seleccionat
				for d in range(self.function.dimensions):
					r = random.random()
					if r <= self.cr:
                        #Assigna a l'individu descendent la dimensió del vector mutant comprovant que
                        #aquesta no excedeix els rangs de cerca
						if mutation[d] < self.function.rangeA:
							offspring.append(self.function.rangeA)
						elif mutation[d] > self.function.rangeB:
							offspring.append(self.function.rangeB)
						else:
							offspring.append(mutation[d])
					else:
                        #Assigna a l'individu descendent la dimensió de l'individu original
						offspring.append(indv[d])
                
                #Avaluam el descendent        
				scoreOffspring = self.function.evalf(offspring)
                #Avaluam el progenitor
				scoreParent = self.function.evalf(indv)
                
                #Afegim a la futura generació el més apte
				if scoreParent < scoreOffspring:
					nextPopulation.append(indv)
				else:
					nextPopulation.append(offspring)
		
			self.population = nextPopulation[:] #Canvi de generació
			
			self.updateResults(self.population) #Actualitzam els resultats
		
            #Comprovam si s'ha optingut el mínim desitjat
			if self.isGoalFounded: break

		self.stopTime = time.time()
        
if __name__ == "__main__":
	run(shubert, 2, -10, 10, seed=32, minimum=-105364, iterations=20000)
	sys.exit(main())

