#!/usr/bin/env python
# encoding: utf-8
"""
Mòdul que implementa la classe EC que representa l'estructura dels algoritmes utilitzats

Created by Joan T. Matamalas Llodrà on 2011-11-11.
Copyright (c) 2011 . All rights reserved.
"""

import random
import time

class Ec(object):
	"""Mètode creador de la classe EC"""
	def __init__(self, function, seed, iterations, threshold = 1e-10, nPop = 50):
		super(Ec, self).__init__()
		self.function = function #Funcio d'avaluacio
		self.seed = seed #Llavor a utilitzar amb el generador pseudoaleatori
		self.iterations = iterations #Nombre d'iteracions màximes
		self.threshold = threshold #Error permès a l'hora de determinar si ha arribat al mínim
		self.nPop = nPop #Tamany de la població de solucions candidates
		self.bestIndv = [] #Millor solució fins el moment
		self.bestFitness = 0 #Fitness de la millor solució
		self.startTime = 0 #Temps de començament
		self.stopTime = 0 #Temps de finalització
		self.iterationsUntilGoal = 0 #Nombre de iteracions afectuades
		self.isGoalFounded = False #S'ha conclos amb èxit l'execució?
		
	def initPopulation(self):
		"""Mètode que inicialitza la població"""
		random.seed(self.seed)#Iniciam el generador pseudoaleatori
		return [[random.uniform(self.function.rangeA,self.function.rangeB) for d in range(self.function.dimensions)] for n in range(self.nPop)]
		
	def getTotalTime(self):
		"""Mètode que retorna el temps total utilitzat"""
		return self.stopTime - self.startTime
		
	def setBestIndv(self, population):
		"""Donada una població, aquest mètode determina el millor individu i el seu fitness"""
		self.bestIndv = min(population, key=self.function.evalf)
		self.bestFitness = self.function.evalf(self.bestIndv)
		
	def updateResults(self, population):
		"""Mètode que actualitza les dades de l'execució donada una població"""
		#Determinam el millor individu de la població
		populationBest = min(population, key=self.function.evalf)
		#Comprovam si és el millor resultat fins el moment, i si procedeix, actualitzam els resultats
		if self.function.evalf(populationBest) < self.bestFitness:
			self.bestIndv = populationBest
			self.bestFitness = self.function.evalf(populationBest)
					
		#Augmentam el nombre d'iteracions	 
		self.iterationsUntilGoal += 1
		
		#Comprovam si hem arribat al mínim cercat
		if self.bestFitness < self.function.minimum+self.threshold:
			self.bestFitness = self.function.minimum
			#Marcam l'execució com exitossa.
			self.isGoalFounded = True
