#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
"""
Mòdul que implementa la classe que representa una execució de l'algoritme GA

Created by Joan T. Matamalas Llodrà on 2011-11-11.
Copyright (c) 2011 . All rights reserved.
"""

import random
import time
import os
import sys
from algorithms.ec import Ec

from functions import *

class Ga(Ec):
	"""Mètode creador de l'objecte"""
	def __init__(self, function, seed, iterations, threshold=1e-10, nPop = 50, mutationRate = 0.02, eliteRate = 0.1):
		super(Ga, self).__init__(function, seed, iterations, threshold, nPop = 50)
		self.mutationRate = mutationRate # Rati de mutació, per defècte 0.02
		self.eliteRate = eliteRate # Rati d'elitisme, per defècte 0.1
		self.population = self.initPopulation() # Població de solucions candidates
		
	def parentSelection(self):
		"""
		Mètode que omple la Mating Pool amb l'ús de la selecció per rànquing i SUS
		"""
	
		#Ordenam els individus depenent de la seva fitness
		popRanked = sorted(self.population, key=self.function.evalf, reverse=True)
	
		#Probabilitat de selecció per rànquing 
		probSelection = (lambda nPop, rank, s = 1.5: ((2-s)/nPop)+((2*rank*(s-1))/(nPop*(nPop-1))))
	
        #Assignam una probabilitat depenent del rànquing a cada individu
		ranking = [[probSelection(len(self.population),rank),popRanked[rank]] for rank in range(len(self.population))]
		
		#Omplint la mating pool utilitzant el SUS
		matingPool = []
		r = random.uniform(0,(1/len(self.population)))
		sumProb = 0
		i = 49

		while len(matingPool) < len(self.population):
			sumProb += ranking[i][0]
			while r <= sumProb:
				matingPool.append(ranking[i][1])
				r += 1.0/len(self.population)
			i -= 1
		return matingPool
			
			
	def crossover(self,matingPool):
		"""
		Mètode que implementa el crossover entre el genotip de dos individus
		"""
		offspring = []
		while len(matingPool) > 0:
            #S'escolleixen dos pares i s'eliminen de la mating pool
			p1Index = random.randint(0,len(matingPool)-1)
			parent1 = matingPool.pop(p1Index)
			p2Index = random.randint(0,len(matingPool)-1)
			parent2 = matingPool.pop(p2Index)
            
            #Es selecciona una dimensió on s'aplicaran els calculs aritmètics
			index = random.randint(0, len(parent1)-1)
            
            #Calculs per a la dimensió seleccionada
			beta = random.uniform(0,1)
			pointParent1 = parent1[index] - beta*(parent1[index]-parent2[index])
			pointParent2 = parent2[index] + beta*(parent1[index]-parent2[index])
            
			tempGenP1 = parent1[0:index]
			tempGenP2 = parent2[0:index]
            
            #S'afegeix la dimensió aritmèticament modificada
			tempGenP1.append(pointParent1)
			tempGenP2.append(pointParent2)
            
            #Intercanvi del genotip dels dos pares per la dimensió previament seleccionada
			offspring.append(tempGenP1+parent2[index+1:])
			offspring.append(tempGenP2+parent1[index+1:])
		return offspring

	def mutate(self,offspring):
		"""
		Mètode que implementa una mutació uniforme sobre nombres reals
		"""
		for n in range(len(offspring)):
			for gen in range(len(offspring[n])):
				if random.random() < self.mutationRate:
                    #Mutació uniforme de un determinat 'gen'
					offspring[n][gen] = random.uniform(self.function.rangeA, self.function.rangeB)

	def run(self):
		"""
		Executa l'algoritme GA
		"""
        
		self.startTime = time.time()

		#Puntuam els individus
		self.setBestIndv(self.population)
	
		for iteration in range(self.iterations):
			#Omplim la mating pool
			matingPool = self.parentSelection()
			#Generam l'offspring
			offspring = self.crossover(matingPool)
			#Procés de mutació sobre els individus.
			self.mutate(offspring)
		
			#Ordenam la població amb relació a la seva fitness
			self.population.sort(key=self.function.evalf)
		
			#Obtenim l'èlit de la generació de pares, tindrem en compte que els 'genotips' han de ser diferents
		
			elitePopulation = []
			for indv in self.population:
				if indv not in elitePopulation:
					elitePopulation.append(indv)
				if len(elitePopulation) >= self.nPop*self.eliteRate:
					break
				
			#Unió de l'èlit i l'offspring per generar, un cop ordenats els individus, la nova generació
			tempPopulation = elitePopulation + offspring
			tempPopulation.sort(key=self.function.evalf)
			self.population = tempPopulation[0:50]
		
			#Guardam els resultats
			self.updateResults(self.population)
		
            #Si s'arriba al mínim dessitjat s'interromp l'execució
			if self.isGoalFounded: break
			
		self.stopTime = time.time()