#!/usr/bin/env python
# encoding: utf-8
"""
Aquest arxiu conté les funcions d'avalauació utilitzades per l'experiment.

Created by Joan T. Matamalas Llodrà on 2011-11-12.
Copyright (c) 2011 . All rights reserved.
"""

import sys
import os
import math
import unittest
import random
from functools import update_wrapper
import functools
import operator
import heapq
import time 

class function(object):
    """Classe que representa els parametres d'una funció determinada"""
    def __init__(self, name, evalf, dimensions, rangeA, rangeB, minimum):
        super(function, self).__init__()
        self.name = name #Nom de la funció
        self.evalf = evalf #Funcio d'avaluació
        self.dimensions = dimensions #Nombre de dimensions
        self.rangeA = rangeA #Interval de cerca
        self.rangeB = rangeB #Interval de cerca
        self.minimum = minimum #Minim teòric

def decorator(d):
    def _d(fn):
        return update_wrapper(d(fn), fn)
    update_wrapper(_d, d)
    return _d

@decorator
def memoize(f):
    """Decorador que implementa una cache dels darrers 20000 calculs realitzats"""
    maxsize = 20000
    current = [0]
    cache = {}  # map from key to value
    heap = []   # list of keys, in LRU heap
    """decorator to 'memoize' a function - caching its results"""
    def _f(*args):
        key = tuple(args[0])
        if not key in cache:
            if current[0] < maxsize:
                current[0] += 1
            else:
                # pop oldest element
                (_,oldkey) = heapq.heappop(heap)
                cache.pop(oldkey)
            # insert this element
            cache[key] = f(*args)
            heapq.heappush(heap,(time.time(),key))
        return cache[key]
    return _f

@memoize
def sphere(x):
	return sum(n*n for n in x)
    
@memoize
def step(x):
	return sum(math.floor(n+0.5)**2 for n in x)

@memoize
def sumSquares(x):
	return sum((i+1)*(x[i]*x[i]) for i in range(len(x)))

@memoize
def colville(x):
	return 100*(pow((pow(x[0],2)-x[1]),2))+pow((x[0]-1),2)+pow((x[2]-1),2)+90*(pow((pow(x[2],2)-x[3]),2))+10.1*(pow((x[1]-1),2)+pow((x[3]-1),2))+19.8*(x[1]-1)*(x[3]-1)

@memoize
def zakharov(x):
	return sum(pow(n,2) for n in x)+pow(sum(0.5*(i+1)*x[i] for i in range(len(x))),2)+pow(sum(0.5*(i+1)*x[i] for i in range(len(x))),4)

@memoize
def himmelblau(x):
    return ((x[0]**2+x[1]-11)**2)+(x[0]+x[1]**2-7)**2

@memoize
def rastrigin(x):
	return sum((pow(n,2)-10*math.cos(2*math.pi*n)+10) for n in x)

@memoize
def schwefel2p22(x):
	return sum(abs(n) for n in x) + functools.reduce(operator.mul, [abs(m) for m in x])

@memoize
def schwefel(x):
	return sum(-n*math.sin(math.sqrt(abs(n))) for n in x)

@memoize
def schaffer(x):
    return 0.5 + (pow(math.sin(math.sqrt(x[0]**2+x[1]**2)),2)-0.5)/((1+0.001*(x[0]**2+x[1]**2))**2)

@memoize
def shubert(x):
    return sum(i*math.cos((i+1)*x[0] + 1) for i in range(1,6))*sum(i*math.cos((i+1)*x[1]+1) for i in range(1,6))


@memoize
def griewank(x):
	return (1./4000)*sum(n*n for n in x)-functools.reduce(operator.mul, [math.cos(x[i]/math.sqrt(i+1)) for i in range(len(x))])+1

@memoize
def ackley(x):
	invN = 1./len(x)
	return -20*math.exp(-0.2*math.sqrt(invN*sum(pow(n,2) for n in x)))-math.exp(invN*sum(math.cos(2*math.pi*m) for m in x)) + 20 + math.exp(1)

@memoize	
def shekel10(x):
	c = [0.1,0.2,0.2,0.4,0.4,0.6,0.3,0.7,0.5,0.5]
	a = [[4,4,4,4],[1,1,1,1],[8,8,8,8],[6,6,6,6],[3,7,3,7],[2,9,2,9],[5,5,3,3],[8,1,8,1],[6,2,6,2],[7,3.6,7,3.6]]
	s = 0
	for j in range(len(a)):
		p = 0
		for i in range(len(x)):
			p += pow(x[i]-a[j][i],2)
		s += 1/(p+c[j])
	return -s
	
@memoize
def michalewicz5(x):
    return -sum(math.sin(x[i])*pow(math.sin(((i+1)*pow(x[i],2))/math.pi),20) for i in range(len(x)))